# LEGAL NOTICE
# This computer software was prepared by Battelle Memorial Institute,
# hereinafter the Contractor, under Contract No. DE-AC05-76RL0 1830
# with the Department of Energy (DOE). NEITHER THE GOVERNMENT NOR THE
# CONTRACTOR MAKES ANY WARRANTY, EXPRESS OR IMPLIED, OR ASSUMES ANY
# LIABILITY FOR THE USE OF THIS SOFTWARE. This notice including this
# sentence must appear on any copies of this computer software.
# 
# EXPORT CONTROL
# User agrees that the Software will not be shipped, transferred or
# exported into any country or used in any manner prohibited by the
# United States Export Administration Act or any other applicable
# export laws, restrictions or regulations (collectively the "Export Laws").
# Export of the Software may require some form of license or other
# authority from the U.S. Government, and failure to obtain such
# export control license may result in criminal liability under
# U.S. laws. In addition, if the Software is identified as export controlled
# items under the Export Laws, User represents and warrants that User
# is not a citizen, or otherwise located within, an embargoed nation
# (including without limitation Iran, Syria, Sudan, Cuba, and North Korea)
#     and that User is not otherwise prohibited
# under the Export Laws from receiving the Software.
# 
# Copyright 2011 Battelle Memorial Institute.  All Rights Reserved.
# Distributed as open-source under the terms of the Educational Community 
# License version 2.0 (ECL 2.0). http://www.opensource.org/licenses/ecl2.php
# 
# For further details, see: http://www.globalchange.umd.edu/models/gcam/
#

# diag_header.R
#
# An automated graphing system to process GCAM output data (as generated by the
# ModelInterface) and generate both standard and user-defined graphs.
#
# This file provides various facilities, including logging, file I/O and settings
#
# Ben Bond-Lamberty, November 2012


# -----------------------------------------------------------------------------
# Load required libraries
if( !require( "rgcam", character.only=T ) ) {
    stop( "Couldn't load rgcam, please install it with devtools::install_github('JGCRI/rgcam')")
}
# NB this diag has been tested with the following libraries:
#   ggplot2     0.9.2-0.9.3
#   reshape2    1.4.2
#   scales      0.2.3
#   plyr        1.8
libs <- c( "ggplot2", "reshape2", "scales", "plyr" )
for( i in libs ) {
    if( !require( i, character.only=T ) ) {
        cat( "Couldn't load", i, "\n" )
        stop( "Use install.packages() to download this library" )
    }
    library( i, character.only=T )
}

# -----------------------------------------------------------------------------
# Global settings (in CAPITALS)
# This first group of settings is protected--we don't want it re-set every time
# this header is read. This first 
if( !exists( "GCAM_SOURCE_FN" ) ) {     # i.e. #ifndef
    GCAM_SOURCE_FN      <- c( "?" )     # name of currently executing source file (stack structure)
    GCAM_LOG_SAVE       <- c( FALSE )   # whether current log is also being saved to file (stack structure)
    GCAM_SOURCE_RD      <- 0            # recursion depth, an index into above structures
    DEPENDENCIES        <- list()       # dependencies (i.e. what files scripts read)
}

# -----------------------------------------------------------------------------
# Behavioral settings
DISPLAY_EACH_GRAPH      <- FALSE
SAVE_EACH_GRAPH         <- TRUE
SAVE_EACH_DATA          <- TRUE
WAIT_EACH_GRAPH         <- FALSE

GRAPH_NUMBER            <- 1

OUTPUT_DIR_ROOT         <- "."
OUTPUT_DIR              <- OUTPUT_DIR_ROOT
OUTPUT_FILETYPE         <- "png"
OUTPUT_FILENAME_SEP     <- "."

HTML_INDEX_TEMPLATE     <- "post_process/index_template.html"
HTML_INDEX_FILE         <- "index.html"
HTML_JS_TEMPLATE        <- "post_process/figures_filter_template.js"
HTML_JS_FILE            <- "figures_filter.js"

GIS_DIR <- "GIS/"
#GIS_FILE <- "GCAM_4"       # This is the detailed shapefile - nice but slow to draw
GIS_FILE <- "GCAM_4_simple" # This is the simple shapefile 

GRAPH_METADATA <- list()    # Keep track of identifying features of each graph generated

LOGLEVEL_DEBUG      <- 4
LOGLEVEL_DETAIL     <- 3
LOGLEVEL_SUMMARY    <- 2
LOGLEVEL_ERROR      <- 1
LOGLEVEL_NONE       <- 0
LOGLEVEL            <- LOGLEVEL_DEBUG

# -----------------------------------------------------------------------------
# Parser settings
GCAM_DATA_COMMENT       <- "#"                          # Comment character for files
SCENARIO_FIELD_NAME     <- "scenario"
UNITS_FIELD_NAME        <- "Units"
TITLE_FIELD_NAME        <- "title"
FILE_FIELD_NAME         <- "file"
TABLE_FIELD_NAME        <- "TABLE"
DATE_FIELD_NAME         <- "date"

# -----------------------------------------------------------------------------
# printlog: time-stamped output
# params: msg (message", " can be many items); ts (add timestamp?), cr (print CR?)
printlog <- function( msg, ..., ts=TRUE, cr=TRUE, level=LOGLEVEL_DEBUG ) {
    if( level <= LOGLEVEL ) {
        if( ts ) cat( date(), GCAM_SOURCE_FN[ GCAM_SOURCE_RD ], "[", GCAM_SOURCE_RD, "]", ": " )
        cat( msg, ... )
        if( cr ) cat( "\n")
    }
}

# -----------------------------------------------------------------------------
# logstart: start a new log (to screen and optionally file)
# params: fn (name of source file being executed), savelog (whether to write to disk)
logstart <- function( fn, savelog=T ) {
    GCAM_SOURCE_RD <<- GCAM_SOURCE_RD + 1       # push
    GCAM_SOURCE_FN[ GCAM_SOURCE_RD ]  <<- fn
    GCAM_LOG_SAVE[ GCAM_SOURCE_RD ] <<- savelog
    logpath <- "logs"
    if( !file.exists( logpath ) )
        dir.create( logpath )
    if( savelog ) sink( paste( logpath, "/", fn, ".log", sep="" ), split=T )
    printlog( "-----" )
    printlog( "Starting", fn )
    
    DEPENDENCIES[[ fn ]] <<- c( NULL ) # Create a new entry in the dependency list
}

# -----------------------------------------------------------------------------
# logstop: stop the current log (to screen and optionally file)
logstop <- function() {

    fn <- GCAM_SOURCE_FN[ GCAM_SOURCE_RD ]

    printlog( "All done with", fn )
    if( GCAM_SOURCE_RD > 0 ) {
        if( GCAM_LOG_SAVE[ GCAM_SOURCE_RD ] ) sink()
        GCAM_SOURCE_RD <<- GCAM_SOURCE_RD - 1       # pop
    } else {
        printlog( "WARNING: Attempt to close a non-open log file")
    }
}
